<body bgcolor="#ffffff" text="#000000" link="#000000" vlink="#000000" >
<center><h1><big>
<: small_caps("Mondo Server") :>
</big></h1></center>
<p>
<hr><p>
<font size=+2>With the help of these fantastic technologies,
<div align=right>
<: extern_href('<img src="/etc/perl_id_bw_sm.gif" border=0 >',
              'http://www.perl.org') :>
<div align=left>
<br>This sophisticated web application was written
<div align=right>
<: extern_href(qq(<img src="/etc/PoweredByOS.gif" border=0 >),
       'http://www.odi.com') :>
<div align=left>
<br>In a mere <b>ten days</b>
<div align=right>
<: extern_href(qq(<img src="/etc/apache_logo.gif" border=0 >),
	       'http://www.apache.org') :>
<div align=left>
<br>With almost zero frustration!

<: sub{
    if (is_cmd('login')) {
	return $ph->x('login');
    }
    0;
} :>
